/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import java.awt.*;
import java.awt.event.*;

import noope.core.Physics;

import noope.output.MessageBox;

import java.lang.Runnable;
import java.lang.Thread;
import java.lang.InterruptedException;


/**
 * This version of the RunDialog uses AWT, rather than Swing.
 *
 * @author Miklos Reiter
 * @version 27-Apr-2001
 */

public class AWTRunDialog extends Dialog implements Runnable {

    private Physics phys; // the Physics object running the simulation

    // I have not got a clue what volatile is, but the Java Reference tells me to make variables
    // used to tell Threads that they have stopped or are suspended volatile.
    private volatile Thread runThread; // the Thread used for running the simulation

    private volatile boolean threadSuspended; // whether the Thread is suspended

    Panel Panel1 = new Panel();
    Button runButton = new Button("Run / Pause");
    Button stopButton = new Button("Stop");
    Label progressLabel = new Label();

    public AWTRunDialog(Frame owner)
    {
	super(owner, "Running simulation...", true);
	try {
	    jbInit();
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    }
    
    private void jbInit() throws Exception {
	this.setLayout(new BorderLayout());
	progressLabel.setText("0.0 s");
	this.add(Panel1, BorderLayout.CENTER);
	Panel1.add(runButton, null);
	Panel1.add(stopButton, null);
	this.add(progressLabel, BorderLayout.SOUTH);
	
	// Run/Pause button
	runButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    runButton_actionPerformed(e);
		}
	    });
	
	// Stop button
	stopButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    stopButton_actionPerformed(e);
		}
	    });
	
	// Make sure, we exit the simulation properly when the dialog is closed
	this.addWindowListener(new WindowAdapter()
	    {
		public void windowClosing(WindowEvent e)
		{
		    runThread = null;
		}
	    });
	this.pack();
	this.setResizable(false);
    }

    /** Sets the Physics object used for the simulation. */
    public void setPhysics(Physics ph)
    {
	phys = ph;
    }

    /** Returns the Physics object used for the simulation. */
    public Physics getPhysics()
    {
	return phys;
    }

    /** Runs the simulation. Use startSimulation() to start it as a separate Thread. */
    public void run()
    {
	Thread simulationThread = Thread.currentThread();
	
	double dt = 0.001;

	if (phys == null)
	    {
		MessageBox.displayMessage((Frame)getOwner(), "Error", "Physics not set yet.");
		return;
	    }

	int i = 0;
	
	for (;;)
	    {
		phys.step(dt);
		if (i++ % 100 == 0)
		    {
			// if we have been told to exit, do so
			if (simulationThread != runThread)
			    {
				// tidy up the dialog and phys
				this.dispose();
				// stop Thread
				return;
			    }
			// while suspended and not stopped, wait
			while (threadSuspended && (simulationThread == runThread))
			    {
				try 
				    {
					simulationThread.sleep(10);
				    }
				catch (InterruptedException e)
				    {}
			    }
			progressLabel.setText(dt*i + " s");
		    }
	    }
    }

    /** Starts the simulation as a separate Thread. */
    public void startSimulation()
    {
	runThread = new Thread(this, "Simulation");
	threadSuspended = false;
	runThread.start();
    }

    /** Stop button. */
    private void stopButton_actionPerformed(ActionEvent e)
    {
	// tell the thread to exit
	runThread = null;
	this.setVisible(false); // close dialog
    }

    /** Run/Pause button. */
    private void runButton_actionPerformed(ActionEvent e)
    {
	// tell the thread that it's suspended or unsuspended
	threadSuspended = !threadSuspended;
    }

    /** Overridden to dispose of phys. */
    public void dispose()
    {
	if (phys != null) phys.dispose();
	super.dispose();
    }
}
