/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import javax.swing.*;
import noope.core.Vector3D;

/**
 * This is an input-verifier for JTextField's containing a colorname or a color-Vector3D with
 * components in the range 0-1.
 *
 * @author Miklos Reiter
 * @version 19-Apr-2001
 *
 */
public class ColorInputVerifier extends InputVerifier
{
    /**
     * Whether the ColorInputVerifier accepts the empty string as a valid
     * input.
     */
    private boolean acceptEmpty;
    
    /**
     * Constructs a new ColorInputVerifier object.
     *
     * This verifier will accept the empty string as a valid entry.
     */
    public ColorInputVerifier() {
	acceptEmpty = true;
    }
    
    /**
     * Constructs a new ColorInputVerifier object.
     *
     * @arg acceptEmpty Whether the empty string should be accepted as
     * valid.
     */
    public ColorInputVerifier(boolean emptyEntryAccepted) {
	acceptEmpty = emptyEntryAccepted;
    }
    
    /**
     * Is the empty string accepted as a valid entry?
     */
    public boolean getAcceptEmpty()
    {
	return acceptEmpty;
    }
    
    /**
     * Sets whether the empty string should be accepted as valid.
     */
    public void setAcceptEmpty(boolean emptyEntryAccepted)
    {
	acceptEmpty = emptyEntryAccepted;
    }
    
    
    /**
     * Displays a message box complaining to the user and informing them
     * briefly of the format of a Color, if the input is invalid, in
     * addition to calling the inherited method.
     */
    public boolean shouldYieldFocus(JComponent input)
    {
	boolean result = super.shouldYieldFocus(input);
	if (!result)
	    JOptionPane.showMessageDialog(input, "Please enter a valid Color string of the format" +
					  " '(x,y,z)' where x, y and z are doubles 0-1 or select" +
					  " an entry from the list.", "Error",
					  JOptionPane.ERROR_MESSAGE);
	return result;
    }

    /**
     * Returns true if the JTextField input contains a valid Color.
     */
    public boolean verify(JComponent input) {
	String s = ((JTextField)input).getText();
	if (getAcceptEmpty() && s.equals("")) return true;
	return validColor(s);
    }

    /** Returns true if the String s represents a valid Color object, false otherwise. */
  private static boolean validColor(String s)
  {
      if (s.equals("black") || (s.equals("blue")) || (s.equals("cyan")) 
	  || (s.equals("darkGray")) || (s.equals("gray")) || (s.equals("green"))
	  || (s.equals("lightGray")) ||  (s.equals("magenta")) || (s.equals("orange"))
	  || (s.equals("pink")) || (s.equals("red")) || (s.equals("white")) ||
	  (s.equals("yellow")))
	  return true; // one of predefined options
      
      try
	  {
	      Vector3D v = new Vector3D(s);
	      return (0.0 <= v.getX() && v.getX() <= 1.0 && 0.0 <= v.getY() && v.getY() <= 1.0
		      && 0.0 <= v.getZ() && v.getZ() <= 1.0);
	      // check values in correct range
	  }
      catch (java.text.ParseException e)
	  {
	      return false; // parsing failed
	  }
  }
}
