/**
 * Copyright (C) 2001  James Kermode (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.core;

// Removed because not supported before Java 1.2
//import java.util.ListIterator;

import noope.laws.Law;
import noope.entities.Entity;
import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;
import noope.input.BREntryNotFoundException;

import noope.input.LawLoader;
import noope.input.DynamicLawLoader;
import noope.input.EntityLoader;
import noope.input.DynamicEntityLoader;
import noope.input.LawPropertyRecordLoader;
import noope.input.DynamicLawPropertyRecordLoader;

/**
 * This is the main wrapper class around Laws and Entities.
 * 
 * It initialises itself from the supplied BlockReader, and constructs all
 * Entity and Law objects, which are stored in private Entities and Laws list
 * objects.
 *
 * @author James Kermode
 * @version 11-May-2001: changed != on Strings to !String.equals(...) because didn't work,
 * Miklos Reiter
 * 28-Apr-2001: removed ListIterator because not supported before Java 1.2
 */
public class Physics
{
    private Entities entities;    // List of entities
    private Laws laws;            // List of laws
    private double time = 0.0;    // Current time in the universe
    
    private LawLoader lawLoader;
    private EntityLoader entityLoader;
    private LawPropertyRecordLoader lawPropertyRecordLoader;
    
   /** Construct a Physics object from the supplied BlockReader, and initialise the Laws
    *  and Entities lists. This constructor creates passes DynamicXXXXLoader objects to what
    *  it constructs.
    *	
    *  @param brPhysics the BlockReader used to initialise private Laws and Entities objects
    *  @exception BRLoadingException if it finds anything wrong with the format of the data 
    *  in the BlockReader passed to it.
    */
    public Physics(BlockReader brPhysics) throws BRLoadingException
    {
	initPhysics(brPhysics, new DynamicLawLoader("noope.laws"), 
		    new DynamicEntityLoader("noope.entities"),
		    new DynamicLawPropertyRecordLoader("noope.output"));
    }

    /**
     * Constructs a new Physics object, just as the other constructor, but you can specify
     * the XXXXLoader objects to use.
     *
     */
    public Physics(BlockReader brPhysics, LawLoader lawL, EntityLoader entL, 
		   LawPropertyRecordLoader lawPRL) throws BRLoadingException
    {
	initPhysics(brPhysics, lawL, entL, lawPRL);
    }

    /** Does the initialization, called by the constructors. */
    private void initPhysics(BlockReader brPhysics, LawLoader lawL, EntityLoader entL,
			     LawPropertyRecordLoader lawPRL)
	throws BRLoadingException
    {
	// set private variables.
	lawLoader = lawL;
	entityLoader = entL;
	lawPropertyRecordLoader = lawPRL;


	// First block must be physics{...}, which must contain laws{...} and entities{...} only.

	// use String.equals, instead of ==; changed by Miklos Reiter
	if (!(brPhysics.getHeader().equals("physics")))
	throw new BRLoadingParseException("Top level block should be physics{...}, but found "+
		                          brPhysics.getHeader(), brPhysics.getBlockLocation());

	// Try to get the laws{...} sub-block, and construct laws from it
	if (!brPhysics.hasMoreBlocks())
	    throw new BRLoadingParseException("End of BlockReader reached before mandatory laws{...} block", 
	                                brPhysics.getBlockLocation());
	BlockReader brLaws = brPhysics.getNextBlock();
	if (!brLaws.getHeader().equals("laws"))
	throw new BRLoadingParseException("First sub-block must be laws{...}, but found "+
			                  brLaws.getHeader(), brPhysics.getBlockLocation());
	laws = new Laws(brLaws, this);

	// Try to get the entities{...} sub-block, and construct entities from it
	if (!brPhysics.hasMoreBlocks())
	    throw new BRLoadingParseException("End of BlockReader reached before mandatory entities{...} block", 
	    brPhysics.getBlockLocation());
	    
	BlockReader brEntities = brPhysics.getNextBlock();
	if (!brEntities.getHeader().equals("entities"))
	throw new BRLoadingParseException("Second sub-block must be entities{...}, but found "+
			                  brEntities.getHeader(), brPhysics.getBlockLocation());
	entities = new Entities(brEntities, this);

	brPhysics.assertEmpty();	
    }

    /** Provide access to the list of Laws */
    public Laws getLaws()
    {
	return laws;
    }
    
    /** Provide access to the list of Entities */
    public Entities getEntities()
    {
        return entities;
    }

    /** Provide access to the current universal time */
    public double getTime()
    {
        return time;
    }
    
   /**
    * Run the simulation for a single step of <i>deltaTime</i>.
    *
    * This method is the main interface between Physics and the program
    * using the library. Calling step runs the simulation forward in time 
    * by a period deltaTime, causing all the Laws to act on all the Entities,
    * and update all their positions and velocities.
    *
    * Rewritten by Miklos Reiter so this method does not use ListIterator any more, for
    * compatibility with Java 1.1.
    *
    * @param deltaTime amount of time to run the simulation for
    * @see noope.entities.Entity#step(double)
    */
    public void step(double deltaTime)
    {
	// for each law in laws: law.act(entities)
	Law law;

	for (int i = 0; i < laws.size(); i++)
	    {
		law = (Law)(laws.elementAt(i));
		law.act();
	    }

	/*for (ListIterator i = laws.listIterator(); i.hasNext(); )
	{
	   law = (Law)i.next();
	   law.act();
	   }*/

	// for each entity in entities: entity.step(deltaTime)
	Entity entity;

	for (int i = 0; i < entities.size(); i++)
	    {
		entity = (Entity)(entities.elementAt(i));
		entity.step(deltaTime);
	    }

	/*for (ListIterator i = entities.listIterator(); i.hasNext(); )
	  {
	  entity = (Entity)i.next();
	  entity.step(deltaTime);
	  }
	*/
	time += deltaTime;
    }
    
    /**
    * Run the simulation for a time <i>duration</i>, in intervals of <i>deltaTime</i>
    *
    * This method resets the internal time to zero, then calls step(deltaTime) repeatedly.
    * Use it when you want to run a simulation for a specific amount of time without
    * doing any processing at each step.
    * @param duration total time to run simulation for
    * @param deltaTime amount of time by step by at each iteration
    */
    public void run(double duration, double deltaTime)
    {
        time = 0;
	while (time <= duration)
	    step(deltaTime);
    }

    /** Disposes of resources by calling dispose() on the list of entities and the list of laws.
	You should call this, not only to release resources more quickly than they would be
	by the garbage collector, but also, in order to tell output laws that they can close
	output devices, such as files, windows, etc. */
    public void dispose()
    {
	if (entities != null)
	    {
		entities.dispose();
		entities = null;
	    }

	if (laws != null)
	    {
		laws.dispose();
		laws = null;
	    }
    }
    
    /** Returns the LawLoader object used by this Physics object. Used by the Laws constructor. */
    public LawLoader getLawLoader()
    {
	return lawLoader;
    }

    /** Returns the EntityLoader object used by this Physics object. 
	Used by the Entities constructor. */
    public EntityLoader getEntityLoader()
    {
	return entityLoader;
    }
    
    /** Returns the LawPropertyRecordLoader object used by this Physics object. Used by certain
	Laws wanting to create LawPropertyRecords. */
    public LawPropertyRecordLoader getLawPropertyRecordLoader()
    {
	return lawPropertyRecordLoader;
    }
}

