/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import java.lang.reflect.Array;

/**
 * This class implements the simplest possible BlockReader.
 * I wrote it because I needed something to test Entity with.
 * 
 * It is called ArrayBlockReader, because the constructor is passed
 * an array of entries and an array of subblocks, and that's how
 * they're stored.
 *
 * N.B.: Locations and contexts are not implemented (and presumably, won't be, because this is
 * only intended for testing).
 *
 * All other methods are overridden and do exactly what they should according
 * to the specification of BlockReader.
 *
 * If you try to read beyond the end of the Entry/Subblock list, you get an
 * Array:IndexOutOfBounds exception.
 * @author Miklos Reiter
 * @version March 21, 2001
 */
public class ArrayBlockReader extends BlockReader
{
    // private data members to store data
    private String header;
    private String parameter;
    private String[][] entries;
    private BlockReader[] subblocks;

    // private data members to store position: index of next item to return
    private int entryPos;
    private int subblockPos;
    
    /** Construct an ArrayBlockReader from strings and string arrays.
    @param h the header of the block
    @param p the parameter of the block.
    @param e is the array of entries; each entry is an array of size 2 of Strings.
    @param sub is the array of subblocks, each of which is a BlockReader.
    */
    public ArrayBlockReader(String h, String p, String[][] e, BlockReader[] sub)
    {
	// store data
	header = h;
	parameter = p;
	entries = e;
	subblocks = sub;

	// set pointers to 0
	entryPos = 0;
	subblockPos = 0;
    }

    public String getHeader()
    {
	return header;
    }

    public String getParameter()
    {
	return parameter;
    }

    public boolean hasMoreEntries()
    {
	if (entries == null) return false;
	return (entryPos < Array.getLength(entries));
    }
    
    public String[] getNextEntry()
    {
	String[] result = peekNextEntry();
	entryPos++;
	return result;
    } 
    
    public String[] peekNextEntry() // as getNextEntry() but does not advance position
    {
	return (entries[entryPos]);	
    }

    public void seekFirstEntry()
    {
	entryPos = 0;
    }

    public BlockReaderLocation getEntryLocation()
    {
	return null;
    }

    public boolean hasMoreBlocks()
    {
	if (subblocks == null) return false;
	return (subblockPos < Array.getLength(subblocks));
    }
    
    public BlockReader getNextBlock()
    {
	BlockReader result = peekNextBlock();
	subblockPos++;
	return result;
    }

    public BlockReader peekNextBlock() // as getNextBlock() but does not advance position
    {
	return subblocks[subblockPos];
    }

    public void seekFirstBlock()
    {
	subblockPos = 0;
    }

    public BlockReaderLocation getBlockLocation()
    {
	return null;
    }

    public BlockReaderContext getContext(BlockReaderLocation loc)
    {
	return null;
    }

   /** This method lists the contents of the BlockReader.
    N.B.: this resets the Entry and Subblock positions to 0 and is only intended for debugging purposes.
    */
    public String toString()
    {
	String result = "";
	String[] brEntry;
	BlockReader subblock;

	// reset Entry/Block positions
	seekFirstEntry();
	seekFirstBlock();

	// first append header information
	result = "\nArrayBlockReader contents:" +
	    "\nHeader: " + header + 
	    "\nParameter: " + parameter + "\n" +

	    // first the entries:
	    "ENTRIES:\n";
	while (hasMoreEntries())
	    {
		brEntry = getNextEntry();
		result += brEntry[0];
		result += " ";
		result += brEntry[1];
		result += "\n";
	    }
	
	// now the subblocks:
	result += "SUBBLOCKS:\n";
	while (hasMoreBlocks())
	    {
		result += getNextBlock();
		result += "\n";
	    }
	
	// now reset positions
	seekFirstEntry();
	seekFirstBlock();

	return result;
    }
}
