/**
 * Copyright (C) 2001  Paul Evans (see the file README for details on how
 * to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

/** This is an abstract class used for reading in the law and
 entity definitions. This might be from a file, but is
 abstract, so it could be from anything, for example,
 a GUI editor program.

 This is a generic class, could be used for anything
 using this block format. (This is the syntax accepted by StringBlockReader.)
 
 <pre>
#any whitespace seperates tokens
#string values can use double quote marks to delimit
#use \" to put a quote mark inside a string
#comments start with a # symbol and extend until end of line
#[] denotes optional

header [parameter]
{
  key1 value1  # this is an entry
  key2 value2
  key3 value3
  ...
  keyN valueN
  subheader1 [subparameter1]	# this is a block
  {
    subkey1 subvalue1
    ...
    subsubheader1 [subsubparameter1]
    {
      ...
    }
  }
  header2 [subparameter2]
  {
    ...
  }
}
 </pre>
 
 The format used for reading in a Physics object is as follows.
<pre> 
# this file is case sensitive
# comments start with hash

physics
{
  laws
  {
    law <class name>
    {
      [name "string"]
      constant1 <data1>
      constant2 <data2>
      ...
    }
  }
  entities
  {
    entity [<class name>]
    {
      [name "string"]
      position <Vector3D>
      velocity <Vector3D>
      inertialMass <double>
    }

    Law1 
    {
      ...
    }
    Law2
    {
      ...
    }
  }
}
</pre>

Here is  an example of using the file format:
<pre>
physics
{
  laws
  {
    law Gravity
    {
      name "Gravity"
      G 6.67e-23
    }
  }
  entities
  {
    entity
    {
    # defaults to Entity class
      name "Electron" position (0.5,54.5,45.1)
      velocity 0	  # promotes to (0,0,0)
      inertialMass 9.1e-31

      Gravity
      {
	mass 9.1e-31
      }
      Electric
      {
	charge -1.6e-19
      }
    }
  }
}
</pre>

 @author Paul Evans
    @version Miklos Reiter 19-May-2001: updated javadoc comments
 Modified by Miklos Reiter 21-Mar-2001: added peekNextEntry 
 and peekNextBlock methods. Modified again by Paul Evans 27-Apr-2001: added toString
 */
public abstract class BlockReader
{
    /** Returns the header of this block. This is in some sense its "title".
     */
  abstract public String getHeader() throws BlockReaderException;

    /** Returns the parameter of this block.
     */
  abstract public String getParameter() throws BlockReaderException;

  /** Returns TRUE if this block has more Entries after
     the one that was last returned */
  abstract public boolean hasMoreEntries() throws BlockReaderException;

  /** Returns the next Entry, as a string array.
  @returns
     [0] entry name
     [1] entry value
     The first time called, it returns the first entry,
     and thereafter, returns successively later entries.
     If no more exist, it will throw an exception.
  */
  abstract public String[] getNextEntry() throws BlockReaderException; 

  /** This method acts as getNextEntry() but does not advance position */
  abstract public String[] peekNextEntry() throws BlockReaderException; 

  /** Moves the "entry pointer" back to the first Entry,
       so that the next call to getNextEntry() will return
       the first one in the block */
  abstract public void seekFirstEntry() throws BlockReaderException;

  /** Returns a BlockReaderLocation object describing the current Entry 
      location. A BlockReaderLocation does not make sense to anyone, except this BlockReader and
      its parents.
  */
  abstract public BlockReaderLocation getEntryLocation() throws BlockReaderException;

  /** Returns TRUE if this block has more Blocks after
     the one that was last returned */
  abstract public boolean hasMoreBlocks() throws BlockReaderException;

  /** Returns the next subblock, which is an instance of BlockReader.
     The first time called, it returns the first entry,
     and thereafter, returns successively later entries.
     If no more exist, it will throw an exception.
  */
  abstract public BlockReader getNextBlock() throws BlockReaderException;

  /** This method acts as getNextBlock() but does not advance position */
  abstract public BlockReader peekNextBlock() throws BlockReaderException;

    /** Resets the subblock position to the beginning, so that the next call to getNextBlock()
	will return the first subblock (unless there are no subblocks, in which case an exception
	is thrown, of course. */
  abstract public void seekFirstBlock() throws BlockReaderException;

  /** Similar to getEntryLocation */
  abstract public BlockReaderLocation getBlockLocation() throws BlockReaderException;

  /** Converts a Location into a fully-qualified Context
       object. This means that error messages, reports, etc... can be 
       generated. 
  */
  abstract public BlockReaderContext getContext(BlockReaderLocation loc) throws BlockReaderException;
	   
    /** Throws exception unless BlockReader is empty. */
  public void assertEmpty() throws BRLoadingException
  {
      if (hasMoreEntries()) {
	  throw new BRLoadingException("Unexpected entries found at end.", getEntryLocation());
      }
      else if (hasMoreBlocks()) {
	  throw new BRLoadingException("Unexpected blocks found at end.", getBlockLocation());
      }
  }
}
