/**
 * Copyright (C) 2001  Paul Evans (see the file README for details on how
 * to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import StringOperations.*;

/**
 * A BlockReader that reads in its data from a String.
 *
 * @author Paul Evans
 * @version June-2001
 */
public class StringBlockReader extends BlockReader
{

    private String fullDefinitionString; // the whole string as given by the user of this class

    private String header; // the header, with escapes converted
    private String parameter; // the parameter, if it exists, with escapes converted
    private String mainData; // the main block data, (ie. data inside { and })
    private int mainDataStartsAt; 
        // the position in the initialisation string given to the constructor at which mainData starts

    private int[] entryPointers = new int[2];
    private int curEntryPointer = 0;

    private int[] blockPointers = new int[2];
    private int curBlockPointer = 0;

    private StringBlockReader topLevel; // reference to top level BlockReader
    private int posFromStart; // position in of toplevel definition string that this instance's defintion string starts

    private final boolean DEBUG_GETNEXT = false; // show debug messages in getNext
    private final boolean DEBUG_ENTRIES = false; // show debug messages in entry methods
    private final boolean DEBUG_BLOCKS = false; // show debug messages in block methods
    private final boolean DEBUG_MISC = false; // show all other debug messages (initialisation, etc...)

    public StringBlockReader(String s)
	throws BlockReaderException
    {
	fullDefinitionString = new String(s); // copy it, don't just take a reference to it.
	topLevel = this;
	posFromStart = 0;
	initialiseToString(s);
    }

    private StringBlockReader(String s, StringBlockReader new_topLevel, int new_posFromStart)
	throws BlockReaderException
    {
	topLevel = new_topLevel;
	posFromStart = new_posFromStart;
	initialiseToString(s);
    }

    private void initialiseToString(String s)
	throws BlockReaderException
    {
	try {
	    StringOperations.verifyString(s); // this doesn't return a result, but throws an exception if there's a problem

	    int splitAt = StringOperations.jumpOverToChar(0, s, '{');

	    if (splitAt >= s.length()) {
		String[] pos = StringOperations.reportPosition(fullDefinitionString, posFromStart);
		BlockReaderContext con = new StringBlockReaderContext(Integer.parseInt(pos[1]), 
								      Integer.parseInt(pos[2]), 
								      pos[0]);
		throw new StringBlockReaderException("Opening brace { not found", 
						     new BlockReaderLocationWithContext(con));
										       
	    }

	    int headStart = StringOperations.jumpOverWhitespace(0, s);
	    int headEnd = StringOperations.jumpOverToken(headStart, s);
	    
	    if (headEnd <= headStart) {
		String[] pos = StringOperations.reportPosition(fullDefinitionString, posFromStart);
		BlockReaderContext con = new StringBlockReaderContext(Integer.parseInt(pos[1]), 
								      Integer.parseInt(pos[2]), 
								      pos[0]);
		throw new StringBlockReaderException("Header not found", 
						     new BlockReaderLocationWithContext(con));
	    }

	    header = StringOperations.convertEscapes(s.substring(headStart, headEnd));

	    int paramStart = StringOperations.jumpOverWhitespace(headEnd, s);
	    int paramEnd = StringOperations.jumpOverToken(paramStart, s);

	    if ((paramEnd <= paramStart) || (paramEnd > splitAt))
		parameter = null;
	    else
		parameter = StringOperations.convertEscapes(s.substring(paramStart, paramEnd));

	    mainDataStartsAt = splitAt + 1;

	    // now cut off end close brace; 

	    // can use splitAt because we've finished with the current value now
	    // we know it must be there because there's a match of open to close brackets, and we know there
	    // is at least one open.
	    splitAt = StringOperations.jumpOverToChar(mainDataStartsAt, s, '}');

	    mainData = s.substring(mainDataStartsAt, splitAt - 1);

	    // that's all folks!
	}
	catch (StringOperationException e) {
	    String[] pos = StringOperations.reportPosition(fullDefinitionString, posFromStart + e.getLocation());
	    BlockReaderContext con = new StringBlockReaderContext(Integer.parseInt(pos[1]),
								  Integer.parseInt(pos[2]),
								  pos[0]);
	    throw new StringBlockReaderException("Error in input string: " + e.getMessage(), 
						 new BlockReaderLocationWithContext(con));
	}

	if (DEBUG_MISC) {
	    System.out.println("header: " + header + "\nparameter: " + parameter);
	    System.out.println("mainData:\n" + mainData);
	}

	seekFirstEntry();
	seekFirstBlock();
    }

    public String getHeader()
    {
	return header;
    }

    public String getParameter()
    {
	return parameter;
    }

    private final byte ITEM_ENTRY = 1;
    private final byte ITEM_BLOCK = 2;

    private int[] getNext(int startAt)
	throws StringOperationException
	/* Returns int[3]:
	 * [0] = type; see ITEM_ENTRY and ITEM_BLOCK
	 * [1] = start pos (points to the exact character where it starts)
	 * [2] = end pos (points to the next character _after_ the item ends)
	 */
	// TODO: make this throw a StringBlockReaderException that has a more accurate location pointer
    {
	int[] ret = {0, -1, -1}; // equals error condition - no more items

	if (DEBUG_GETNEXT) 
	    System.out.println("getNext startAt=" + startAt);

	// go to first non-space character
	ret[1] = StringOperations.jumpOverWhitespace(startAt, mainData);

	if (DEBUG_GETNEXT) 
	    System.out.println("        ret[1]=" + ret[1]);

	if (ret[1] < 0) return ret; // deal with error

	// find end of this token
	int posOfSplit = StringOperations.jumpOverToken(ret[1], mainData);

	if (DEBUG_GETNEXT) 
	    System.out.println("        posOfSplit=" + posOfSplit);

	if (posOfSplit < 0) {
	    return ret; // deal with more error
	}

	// find start of next token or "{" character
	int posOfStart2 = StringOperations.jumpOverWhitespace(posOfSplit, mainData);

	if (DEBUG_GETNEXT) 
	    System.out.println("        posOfStart2=" + posOfStart2);

	if (posOfStart2 < 0) // found a single token at end of definition string - i.e. an entry with no value
	    throw new StringOperationException("Found entry name with no value", ret[1]);

	if ((mainData.charAt(posOfStart2) != '{')) {

	    ret[2] = StringOperations.jumpOverToken(posOfStart2, mainData);
	    
	    // check if we've not gone off the edge yet...
	    if (ret[2] < mainData.length()) {
		// ... if not go to start of next token so we can check if it's a block with a parameter.
		int startOfNextToken = StringOperations.jumpOverWhitespace(ret[2], mainData);
		
		if ((startOfNextToken < mainData.length()) && (mainData.charAt(startOfNextToken) == '{')) {
		    if (DEBUG_GETNEXT) 
			System.out.println("        is actually a block");
		    posOfStart2 = startOfNextToken;
		}
	    }
	}

	// now see what we've got 
	if (mainData.charAt(posOfStart2) == '{') {
	    // got a block with no param - so find end of it
	    ret[0] = ITEM_BLOCK;
	    ret[2] = StringOperations.jumpOverToken(posOfStart2, mainData); 
	    // this will actually jump through the entire block, since it knows about structured braces { and }
	}
	else {
	    // got an entry
	    ret[0] = ITEM_ENTRY;
	}

	return ret;
    }

    private String[] doNextEntry(boolean move, boolean requirereturn)
	throws BlockReaderException
    {
	String[] ret = {"",""};

	if (!hasMoreEntries())
	    throw new StringBlockReaderException("No more entries", null);

	if (requirereturn) {
	    if (DEBUG_ENTRIES) 
		System.out.println("entryPointers={" + entryPointers[0] + "," + entryPointers[1] + "}");
	    String innerString = mainData.substring(entryPointers[0], entryPointers[1]);
	    if (DEBUG_ENTRIES) 
		System.out.println("innerString=\"" + innerString + "\"");
	    try {
		int nameEndsAt = StringOperations.jumpOverToken(0, innerString);
		int valueBeginsAt = StringOperations.jumpOverWhitespace(nameEndsAt, innerString);

		ret[0] = StringOperations.convertEscapes(innerString.substring(0, nameEndsAt));
		ret[1] = StringOperations.convertEscapes(innerString.substring(valueBeginsAt));

		curEntryPointer = entryPointers[0];
	    }
	    catch (StringOperationException e) {
		throw new StringBlockReaderException("Error in input string: " + e.getMessage(), 
						     new StringBlockReaderLocation(topLevel, posFromStart + mainDataStartsAt + e.getLocation()));
	    }
	}

	if (move) {
	    int[] pos = {-1, 0, entryPointers[1] - 1};

	    if (DEBUG_ENTRIES) 
		System.out.println("doNextEntry: started with pos={" + pos[0] + "," + pos[1] + "," + pos[2] + "}");

	    try {
		while ((pos[0] != ITEM_ENTRY) && (pos[0] != 0)) pos = getNext(pos[2] + 1);
	    }
	    catch (StringOperationException e) {
		throw new StringBlockReaderException("Error in input string: " + e.getMessage(), 
						     new StringBlockReaderLocation(topLevel, posFromStart + mainDataStartsAt + e.getLocation()));
	    }

	    if (DEBUG_ENTRIES) 
		System.out.println("doNextEntry: ended   with pos={" + pos[0] + "," + pos[1] + "," + pos[2] + "}");

	    entryPointers[0] = pos[1];
	    entryPointers[1] = pos[2];
	}

	return ret;
    }

    public boolean hasMoreEntries()
    {
	return (entryPointers[0] >= 0) && (entryPointers[1] >= 0);
    }

    public String[] getNextEntry() 
	throws BlockReaderException
    {
	if (DEBUG_ENTRIES) System.out.println("Getting next entry");
	return doNextEntry(true, true);
    }

    public String[] peekNextEntry() 
	throws BlockReaderException
    {
	if (DEBUG_ENTRIES) System.out.println("Peeking at next entry");
	return doNextEntry(false, true);
    }

    public void seekFirstEntry()
	throws BlockReaderException
    {
	entryPointers[1] = 0;
	if (DEBUG_ENTRIES) System.out.println("Seeking first entry...");

	// to update the position ready for first entry, or -1 if none
	doNextEntry(true, false);
    }

    public BlockReaderLocation getEntryLocation()
    {
	return new StringBlockReaderLocation(topLevel, posFromStart + mainDataStartsAt + curEntryPointer);
    }

    private BlockReader doNextBlock(boolean move, boolean requirereturn)
	throws BlockReaderException
    {
	BlockReader ret = null;

	if (!hasMoreBlocks())
	    throw new StringBlockReaderException("No more blocks", null);

	if (requirereturn) {
	    if (DEBUG_BLOCKS) 
		System.out.println("blockPointers={" + blockPointers[0] + "," + blockPointers[1] + "}");
	    String innerString = mainData.substring(blockPointers[0], blockPointers[1]);
	    if (DEBUG_BLOCKS)
		System.out.println("innerString=\"" + innerString + "\"");
	    ret = new StringBlockReader(innerString, 
					topLevel, 
					posFromStart + mainDataStartsAt + blockPointers[0]);
	    curBlockPointer = blockPointers[0];
	}

	if (move) {
	    
	    int[] pos = {-1, 0, blockPointers[1] - 1};

	    if (DEBUG_BLOCKS) 
		System.out.println("doNextBlock: started with pos={" + pos[0] + "," + pos[1] + "," + pos[2] + "}");

	    try {
		while ((pos[0] != ITEM_BLOCK) && (pos[0] != 0)) pos = getNext(pos[2] + 1);
	    }
	    catch (StringOperationException e) {
		throw new StringBlockReaderException("Error in input string: " + e.getMessage(), 
						     new StringBlockReaderLocation(topLevel, posFromStart + mainDataStartsAt + e.getLocation()));
	    }
	
	    if (DEBUG_BLOCKS) 
		System.out.println("doNextBlock: ended   with pos={" + pos[0] + "," + pos[1] + "," + pos[2] + "}");

	    blockPointers[0] = pos[1];
	    blockPointers[1] = pos[2];
	}

	return ret;
    }

    public boolean hasMoreBlocks()
    {
	return (blockPointers[0] >= 0) && (blockPointers[1] >= 0);
    }

    public BlockReader getNextBlock()
	throws BlockReaderException
    {
	if (DEBUG_BLOCKS) System.out.println("Getting next block");
	return doNextBlock(true, true);
    }

    public BlockReader peekNextBlock()
	throws BlockReaderException
    {
	if (DEBUG_BLOCKS) System.out.println("Peeking at next block");
	return doNextBlock(false, true);
    }

    public void seekFirstBlock()
	throws BlockReaderException
    {
	blockPointers[1] = 0;
	if (DEBUG_BLOCKS) System.out.println("Seeking to first block");

	// to update the position ready for first block, or -1 if none
	doNextBlock(true, false);
    }

    public BlockReaderLocation getBlockLocation()
    {
	return new StringBlockReaderLocation(topLevel, posFromStart + mainDataStartsAt + curBlockPointer);
    }

    public BlockReaderContext getContext(BlockReaderLocation loc)
    {

	System.err.println("WARNING: StringBlockReader.getContext(BlockReaderLocation) is now depricated.\n" +
			   "please use BlockReaderLocation.convertToContext() instead.");

	if (loc instanceof StringBlockReaderLocation) 
	    return topLevel.convertContext(((StringBlockReaderLocation)loc).getPosFromStart());
	else 
	    return null;
    }

    // package visibility so StringBlockReaderLocation can see it
    StringBlockReaderContext convertContext(int loc)
    {
	// I know I'm in top level here...
	String[] position = StringOperations.reportPosition(fullDefinitionString, loc);

	return new StringBlockReaderContext(Integer.parseInt(position[1]), 
					    Integer.parseInt(position[2]), 
					    position[0]);
    }
}
