/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.laws;

import java.lang.NumberFormatException;
import java.lang.Double;

import noope.core.*;

import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;
import noope.input.BREntryNotFoundException;

import noope.entities.Entity;

/**
  This is the LawPropertyRecord class associated with the Gravity law.
  The only thing it stores is the gravitationalMass (a double).
  
  @author Miklos Reiter
  @version March 21 2001
 */
public class GravityPropertyRecord extends LawPropertyRecord
{
    private double gravitationalMass;
    
    /** Construct the GravityPropertyRecord from the supplied BlockReader,
       for the supplied Entity. Expects the first entry in br to be "gravitationalMass", 
       followed by a value that can be interpreted as a double. If it doesn't find this 
       entry, it sets gravitationalMass to the inertialMass of ent, unless ent is null, in which
       case it raises an Exception. 
       @param br BlockReader used to initialise GravityPropertyRecord
       @param ent Entity for which the GravityPropertyRecord is being constructed */
    public GravityPropertyRecord(BlockReader br, Entity ent) throws BRLoadingException
    {
        super(br, ent);

        String[] brEntry; // store an entry from the BlockReader here temporarily

        // just peek entry first, for what if we are not supposed to read this...
        if ((br != null) && br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("gravitationalMass")))
            {
                try
                    {
                        gravitationalMass = Double.valueOf(brEntry[1]).doubleValue();
                        br.getNextEntry(); // gravitationalMass successfully read? then advance pointer
                    }
                catch (NumberFormatException e)
                    {
			throw new BRLoadingParseException("Error while parsing for double!", br.getEntryLocation());
                    }
            }
        else if (ent != null) // try to get inertialMass of ent
            {
                gravitationalMass = ent.getInertialMass();
            }
        else // we can't construct the LawPropertyRecord
            {
                throw new BREntryNotFoundException("Entry 'gravitationalMass' not found", br.getEntryLocation());
            }
    }


    /** At this point, I've gone to have tea, so you'll have to guess, what this method does.*/
    public double getGravitationalMass()
    {
	return gravitationalMass;
    }
    
    /** For debug-quality printing. */
    public String toString()
    {
	return "gravitationalMass: " + gravitationalMass;
    }
}
