/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import noope.core.Vector3D;

import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;

import noope.entities.Entity;

import java.text.ParseException;

import java.awt.Color;
import java.awt.Graphics2D;

import java.lang.Double;

/** This is an abstract subclass of OutputProjectionPropertyRecord implementing a color property.
    This LawPropertyRecord stores one color and is suitable for objects that you want to draw in
    one color.
    @author Miklos Reiter
    @version 30 March 2001
 */
abstract public class OutputProjectionColoredPropertyRecord extends OutputProjectionPropertyRecord
{
    /** The default color used to draw the object if none is given. */
    public static Color defaultColor = Color.black;

    /** This field stores the color used to draw the object. */
    protected Color color; // the color in which this LawPropertyRecord is to be drawn.

    protected double oldX = java.lang.Double.NaN, oldY = java.lang.Double.NaN;

    /** Constructs a new OutputProjectionPropertyRecord object from the BlockReader br and
	for the Entity ent. The BlockReader can be empty or even null, in which case
	defaultColor is used. It may contain the
	key "colorName" with String value in {black, blue, cyan, darkGray, gray, green, lightGray,
	magenta, orange, pink, red, white, yellow} *or* the key "colorRGB" which has to be a
	Vector3D object (i.e. a String of the form "(x, y, z)") with x, y and z doubles in the
	range 0-1.
    */
    public OutputProjectionColoredPropertyRecord(BlockReader br, Entity ent) 
	throws BRLoadingException
    {
	super(br, ent);

	String[] brEntry; // for reading entries from br
	
	if (br != null && br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("colorName")))
	    {
		br.getNextEntry(); // advance pointer
		color = lookupColor(brEntry[1], br);
	    }
	else // if nothing is given try the other possibility
	    {
		if (br != null && br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("colorRGB")))
		    {
			try 
			    {
				br.getNextEntry(); // advance pointer
				Vector3D v = new Vector3D(brEntry[1]);
				float x = (float)v.getX(), y = (float)v.getY(), z = (float)v.getZ();
				if (0.0 <= x && x <= 1.0 && 0.0 <= y && y <= 1.0 && 0.0 <= z &&
				    z <= 1.0)
				    {
					color = new Color(x, y, z);
				    }
				else
				    {
					throw new BRLoadingException("RGB-values for color should "
								     + "be between 0 and 1",
								     br.getEntryLocation());
				    }
			    }
			catch (ParseException e) 
			    {
				throw new BRLoadingParseException("Failed to parse for Vector3D",
								  br.getEntryLocation());
			    }
		    }
		else
		    {
			color = defaultColor; // fall back on default
		    }
	    }
    }


    /** Tries to look up the color named in s. If it fails, i.e. if the string is different
	from all the ones enumerated in the description of the constructor, then it throws
	a BRLoadingException. */
    protected static Color lookupColor(String s, BlockReader br) throws BRLoadingException
    {
	// If we wanted efficiency, we could implement a Hashtable, or even check the first letter
	// first and then see which of the alternatives the name really matched. However, I don't
	// think it's worth the pain.
	if (s.equals("black")) return Color.black; else if
	   (s.equals("blue")) return Color.blue; else if
	   (s.equals("cyan")) return Color.cyan; else if
	   (s.equals("darkGray")) return Color.darkGray; else if
	   (s.equals("gray")) return Color.gray; else if
	   (s.equals("green")) return Color.green; else if
           (s.equals("lightGray")) return Color.lightGray; else if
	   (s.equals("magenta")) return Color.magenta; else if
	   (s.equals("orange")) return Color.orange; else if
	   (s.equals("pink")) return Color.pink; else if
	   (s.equals("red")) return Color.red; else if
      	   (s.equals("white")) return Color.white; else if
           (s.equals("yellow")) return Color.yellow;
	else
	    throw new BRLoadingException("I don't know the color " + s + "!",
					 br.getEntryLocation());
    }
}
