/**
 * Copyright (C) 2001  James Kermode (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import java.lang.Integer;
import java.util.ListIterator;
import java.io.*;

import noope.core.*;
import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;
import noope.input.BREntryNotFoundException;

import noope.entities.Entity;
import noope.laws.Law;

/**
   This is a descendant of Law which writes the position of all entities to
   a text file every so often. The output file format is compatible with
   povray:
  
   <pre>
   &lt;x1, y1, z2&gt;, &lt;x2, y2, z2&gt;, &lt;x3, y3, z3&gt;...
   ...
   ...
   </pre>
 
   Parameters are read from a BlockReader, in this format:
   
   <pre>
   law OutputText
   {
       file		<string>
       interval	<int>
   }
   </pre>
   
   <i>file</i> gives the file to which output will be written, and 
   <i>interval</i> gives the number of calls to act() that are ignored 
   between producing output.
   
   @author James Kermode 
   @version 22-Mar-2001
 */
public class OutputText extends Law
{
    private String fileName;
    private int interval, actCalls = 0;
    private FileWriter writer;
    
    public OutputText(BlockReader br, int lawnumber, Physics phys) throws BRLoadingException
    {
        super(br, lawnumber, phys);
	
	String[] brEntry;
	
	// Get first parameter: file
        if (br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("file")))
        {
       	    fileName = brEntry[1];
            br.getNextEntry(); // if we need the entry, advance pointer in br
	}
        else
        {
            throw new BREntryNotFoundException("File not specified", br.getEntryLocation());
        }
	
	// Try to open the file for writing
	try
	{
	    writer = new FileWriter(fileName);
	    // write the header row
        }
	catch (IOException e)
	{
	    throw new BREntryNotFoundException(e.getMessage(), br.getEntryLocation());
	}
	
	// Get second parameter: interval
	if (br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0].equals("interval")))
        {
            try
	    {
	        interval = Integer.parseInt(brEntry[1]);
                br.getNextEntry(); // if we need the entry, advance pointer in br
            }
            catch (NumberFormatException e)
            {
                 throw new BRLoadingParseException("Error while parsing interval",
		                                   br.getEntryLocation());
            }
	}
        else
        {
            throw new BREntryNotFoundException("Interval not specified", br.getEntryLocation());
        }
    }
    
    public void act()
    {
        // Quit except when actCalls == interval
	if (actCalls++ != interval) return;
	    
	actCalls = 0; // Reset actCalls
        Entities entities = physics.getEntities();
	Entity entity;
	Vector3D pos;
	
	try
	{
            // Print the current time
	    writer.write(physics.getTime() + ", ");	
	
	    // For each entity in entities, print the it's position in the format: <x, y, z>
	    for (ListIterator i = entities.listIterator(); i.hasNext(); )
	    {
		entity = (Entity)i.next();
        	pos = entity.getPosition();
		writer.write("<"+pos.getX()+", "+
		        	 pos.getY()+", "+
		        	 pos.getZ()+">, ");
	    }
            writer.write("\n");
	    writer.flush();
        }
	catch (IOException e)
	{
	    System.out.println("TODO: Error in OutputFile.act(): "+e.getMessage());
	    System.exit(1);
	}
    }
}    
